package com.prodega.proshop.integration.spi;

import java.io.PrintWriter;
import java.io.Serializable;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.security.auth.Subject;

import org.apache.log4j.Logger;

public class GenericManagedConnectionFactory implements ManagedConnectionFactory, Serializable {

    private static final long serialVersionUID = 1L;

    private static final Logger LOGGER = Logger.getLogger(GenericManagedConnectionFactory.class);

    private PrintWriter out;

    private Date creationDate = new Date();

    public GenericManagedConnectionFactory() {
        out = new PrintWriter(System.out);
        LOGGER.debug("GenericManagedConnectionFactory.constructor");
    }

    public Object createConnectionFactory(ConnectionManager cxManager) throws ResourceException {
        LOGGER.debug("GenericManagedConnectionFactory.createConnectionFactory,1");
        return new IODataSourceImpl(out, this, cxManager);
    }

    public Object createConnectionFactory() throws ResourceException {
        LOGGER.debug("GenericManagedConnectionFactory.createManagedFactory,2");
        return new IODataSourceImpl(out, this, null);
    }

    public ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo info) {
        LOGGER.debug("GenericManagedConnectionFactory.createManagedConnection");
        return new GenericManagedConnection(out, this, info);
    }

    public ManagedConnection matchManagedConnections(@SuppressWarnings("rawtypes") Set connectionSet,
            Subject subject, ConnectionRequestInfo info) throws ResourceException {
        LOGGER.debug("GenericManagedConnectionFactory.matchManagedConnections Subject " + subject
                + " Info: " + info);
        for (Iterator<?> it = connectionSet.iterator(); it.hasNext();) {
            GenericManagedConnection gmc = (GenericManagedConnection) it.next();
            ConnectionRequestInfo connectionRequestInfo = gmc.getConnectionRequestInfo();
            if ((info == null) || connectionRequestInfo.equals(info)) {
                return gmc;
            }
        }
        throw new ResourceException("Cannot find connection for info!");
    }

    public void setLogWriter(PrintWriter out) throws ResourceException {
        LOGGER.debug("GenericManagedConnectionFactory.setLogWriter");
        this.out = out;
    }

    public PrintWriter getLogWriter() throws ResourceException {
        LOGGER.debug("GenericManagedConnectionFactory.getLogWriter");
        return this.out;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((creationDate == null) ? 0 : creationDate.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        GenericManagedConnectionFactory other = (GenericManagedConnectionFactory) obj;
        if (creationDate == null) {
            if (other.creationDate != null) {
                return false;
            }
        } else if (!creationDate.equals(other.creationDate)) {
            return false;
        }
        return true;
    }

}
