package com.prodega.proshop.integration.spi;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import org.apache.log4j.Logger;

import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.PdfCopyFields;
import com.itextpdf.text.pdf.PdfReader;
import com.prodega.proshop.integration.exceptions.IOTransactionException;

public class PrintAction implements TransactionalAction {

    private final static Logger LOGGER = Logger.getLogger(PrintAction.class);

    private String printer;
    private String tempFileName;
    private byte[] content;
    private int count;

    public PrintAction(String printer, String tempFileName, byte[] content, int count) {
        super();
        this.printer = printer;
        this.tempFileName = tempFileName;
        this.content = content;
        this.count = count;
    }

    @Override
    public void commit() {
        String execString = System.getenv("LPR");
        String executable = execString == null ? "lpr" : execString;
        try {
            FileOutputStream fileOutputStream = new FileOutputStream(tempFileName, false);
            fileOutputStream.write(content);
            fileOutputStream.close();

            String cmd = String.format("%s -P%s -#%d -o Collate=True %s", executable, printer, count,
                    tempFileName);
            Process p = Runtime.getRuntime().exec(cmd);
            int exitCode = p.waitFor();
            if (exitCode != 0) {
                if (execString != null) {
                    LOGGER.warn("lpr exited with status " + exitCode);
                } else {
                    throw new IOTransactionException("lpr exited with status " + exitCode);
                }
            }
        } catch (IOException ex) {
            throw new IOTransactionException(ex);
        } catch (InterruptedException ex) {
            throw new IOTransactionException(ex);
        } finally {
            new File(tempFileName).delete();
        }
    }

    @Override
    public void rollback() {
        new File(tempFileName).delete();
    }

    public String getPrinter() {
        return printer;
    }

    public void setPrinter(String printer) {
        this.printer = printer;
    }

    public String getTempFileName() {
        return tempFileName;
    }

    public void setTempFileName(String tempFileName) {
        this.tempFileName = tempFileName;
    }

    public byte[] getContent() {
        return content;
    }

    public void setContent(byte[] content) {
        this.content = content;
    }

    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }

    public void addPdfContent(byte[] content2, int count2) throws IOException {
        OutputStream os = null;
        PdfCopyFields cp = null;
        PdfReader r1 = null;
        PdfReader r2 = null;
        try {
            os = new ByteArrayOutputStream();
            cp = new PdfCopyFields(os);
            r1 = new PdfReader(content);
            r2 = new PdfReader(content2);
            for (int i = 0; i < count; i++) {
                cp.addDocument(r1);
            }
            for (int i = 0; i < count2; i++) {
                cp.addDocument(r2);
            }
            cp.getWriter().close();
            cp.close();
            r1.close();
            r2.close();
            os.close();
            count = 1;
            content = ((ByteArrayOutputStream) os).toByteArray();
        } catch (DocumentException e) {
            throw new IOException(e.getMessage());
        } catch (IOException e) {
            throw e;
        } finally {
            if (cp != null) {
                cp.getWriter().close();
                cp.close();
            }
            if (r1 != null) {
                r1.close();
            }
            if (r2 != null) {
                r2.close();
            }
        }
    }
}
